@@ -33,10 +33,10 @@ import fixtures
 import testscenarios
 
 from pbr import packaging
-from pbr import tests
+from pbr.tests import base
 
 
-class EmailTestCase(tests.BaseTestCase):
+class EmailTestCase(base.BaseTestCase):
 
     def test_str_dict_replace(self):
         string = 'Johnnie T. Hozer'
@@ -45,34 +45,33 @@ class EmailTestCase(tests.BaseTestCase):
                          packaging.canonicalize_emails(string, mapping))
 
 
-class MailmapTestCase(tests.BaseTestCase):
+class MailmapTestCase(base.BaseTestCase):
 
     def setUp(self):
         super(MailmapTestCase, self).setUp()
-        self.git_dir = self.useFixture(fixtures.TempDir()).path
-        self.mailmap = os.path.join(self.git_dir, '.mailmap')
+        self.root_dir = self.useFixture(fixtures.TempDir()).path
+        self.mailmap = os.path.join(self.root_dir, '.mailmap')
 
     def test_mailmap_with_fullname(self):
-        print(self.mailmap, self.git_dir)
         with open(self.mailmap, 'w') as mm_fh:
             mm_fh.write("Foo Bar <email@foo.com> Foo Bar <email@bar.com>\n")
         self.assertEqual({'<email@bar.com>': '<email@foo.com>'},
-                         packaging.read_git_mailmap(self.git_dir))
+                         packaging.read_git_mailmap(self.root_dir))
 
     def test_mailmap_with_firstname(self):
         with open(self.mailmap, 'w') as mm_fh:
             mm_fh.write("Foo <email@foo.com> Foo <email@bar.com>\n")
         self.assertEqual({'<email@bar.com>': '<email@foo.com>'},
-                         packaging.read_git_mailmap(self.git_dir))
+                         packaging.read_git_mailmap(self.root_dir))
 
     def test_mailmap_with_noname(self):
         with open(self.mailmap, 'w') as mm_fh:
             mm_fh.write("<email@foo.com> <email@bar.com>\n")
         self.assertEqual({'<email@bar.com>': '<email@foo.com>'},
-                         packaging.read_git_mailmap(self.git_dir))
+                         packaging.read_git_mailmap(self.root_dir))
 
 
-class SkipFileWrites(tests.BaseTestCase):
+class SkipFileWrites(base.BaseTestCase):
 
     scenarios = [
         ('changelog_option_true',
@@ -136,7 +135,7 @@ class SkipFileWrites(tests.BaseTestCase):
              or self.env_value is not None))
 
 
-class GitLogsTest(tests.BaseTestCase):
+class GitLogsTest(base.BaseTestCase):
 
     def setUp(self):
         super(GitLogsTest, self).setUp()
@@ -171,23 +170,20 @@ class GitLogsTest(tests.BaseTestCase):
         with open(os.path.join(self.temp_path, "ChangeLog"), "r") as ch_fh:
             self.assertTrue("email@foo.com" in ch_fh.read())
 
-    def _fake_log_output(self, cmd, mapping):
-        for (k, v) in mapping.items():
-            if cmd.startswith(k):
-                return v.encode('utf-8')
-        return b""
-
     def test_generate_authors(self):
-        author_old = "Foo Foo <email@foo.com>"
-        author_new = "Bar Bar <email@bar.com>"
-        co_author = "Foo Bar <foo@bar.com>"
-        co_author_by = "Co-authored-by: " + co_author
+        author_old = u"Foo Foo <email@foo.com>"
+        author_new = u"Bar Bar <email@bar.com>"
+        co_author = u"Foo Bar <foo@bar.com>"
+        co_author_by = u"Co-authored-by: " + co_author
 
-        git_log_cmd = ("git --git-dir=%s log --format" % self.git_dir)
-        git_co_log_cmd = ("git log --git-dir=%s" % self.git_dir)
+        git_log_cmd = (
+            "git --git-dir=%s log --format=%%aN <%%aE>" % self.git_dir)
+        git_co_log_cmd = ("git --git-dir=%s log" % self.git_dir)
+        git_top_level = "git rev-parse --show-toplevel"
         cmd_map = {
             git_log_cmd: author_new,
             git_co_log_cmd: co_author_by,
+            git_top_level: self.root_dir,
         }
 
         exist_files = [self.git_dir,
@@ -196,10 +192,12 @@ class GitLogsTest(tests.BaseTestCase):
             "os.path.exists",
             lambda path: os.path.abspath(path) in exist_files))
 
-        self.useFixture(fixtures.FakePopen(lambda proc_args: {
-            "stdout": BytesIO(
-                self._fake_log_output(proc_args["args"][2], cmd_map))
-        }))
+        def _fake_run_shell_command(cmd, **kwargs):
+            return cmd_map[" ".join(cmd)]
+
+        self.useFixture(fixtures.MonkeyPatch(
+            "pbr.packaging._run_shell_command",
+            _fake_run_shell_command))
 
         with open(os.path.join(self.temp_path, "AUTHORS.in"), "w") as auth_fh:
             auth_fh.write("%s\n" % author_old)
@@ -214,7 +212,7 @@ class GitLogsTest(tests.BaseTestCase):
             self.assertTrue(co_author in authors)
 
 
-class BuildSphinxTest(tests.BaseTestCase):
+class BuildSphinxTest(base.BaseTestCase):
 
     scenarios = [
         ('true_autodoc_caps',
@@ -240,7 +238,7 @@ class BuildSphinxTest(tests.BaseTestCase):
         pkg_fixture = fixtures.PythonPackage(
             "fake_package", [("fake_module.py", b"")])
         self.useFixture(pkg_fixture)
-        self.useFixture(tests.DiveDir(pkg_fixture.base))
+        self.useFixture(base.DiveDir(pkg_fixture.base))
 
     def test_build_doc(self):
         if self.has_opt:
@@ -256,7 +254,7 @@ class BuildSphinxTest(tests.BaseTestCase):
                 "api/fake_package.fake_module.rst") == self.has_autodoc)
 
 
-class ParseRequirementsTest(tests.BaseTestCase):
+class ParseRequirementsTest(base.BaseTestCase):
 
     def setUp(self):
         super(ParseRequirementsTest, self).setUp()
@@ -333,7 +331,7 @@ class ParseRequirementsTest(tests.BaseTestCase):
                          packaging.parse_requirements([self.tmp_file]))
 
 
-class ParseDependencyLinksTest(tests.BaseTestCase):
+class ParseDependencyLinksTest(base.BaseTestCase):
 
     def setUp(self):
         super(ParseDependencyLinksTest, self).setUp()
